@@ -1,10 +1,30 @@
 
 /* Generated data (by glib-mkenums) */
 
-#include "gdata.h"
+#include "gdata-service.h"
+#include "gdata-parsable.h"
 #include "gdata-enums.h"
 
-/* enumerations from "gdata-service.h" */
+/* enumerations from "./gdata/gdata-service.h" */
+GType
+gdata_operation_type_get_type (void)
+{
+  static GType etype = 0;
+  if (etype == 0) {
+    static const GEnumValue values[] = {
+      { GDATA_OPERATION_QUERY, "GDATA_OPERATION_QUERY", "query" },
+      { GDATA_OPERATION_INSERTION, "GDATA_OPERATION_INSERTION", "insertion" },
+      { GDATA_OPERATION_UPDATE, "GDATA_OPERATION_UPDATE", "update" },
+      { GDATA_OPERATION_DELETION, "GDATA_OPERATION_DELETION", "deletion" },
+      { GDATA_OPERATION_DOWNLOAD, "GDATA_OPERATION_DOWNLOAD", "download" },
+      { GDATA_OPERATION_UPLOAD, "GDATA_OPERATION_UPLOAD", "upload" },
+      { GDATA_OPERATION_AUTHENTICATION, "GDATA_OPERATION_AUTHENTICATION", "authentication" },
+      { 0, NULL, NULL }
+    };
+    etype = g_enum_register_static ("GDataOperationType", values);
+  }
+  return etype;
+}
 GType
 gdata_service_error_get_type (void)
 {
@@ -13,18 +33,14 @@ gdata_service_error_get_type (void)
     static const GEnumValue values[] = {
       { GDATA_SERVICE_ERROR_UNAVAILABLE, "GDATA_SERVICE_ERROR_UNAVAILABLE", "unavailable" },
       { GDATA_SERVICE_ERROR_PROTOCOL_ERROR, "GDATA_SERVICE_ERROR_PROTOCOL_ERROR", "protocol-error" },
-      { GDATA_SERVICE_ERROR_WITH_QUERY, "GDATA_SERVICE_ERROR_WITH_QUERY", "with-query" },
       { GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED, "GDATA_SERVICE_ERROR_ENTRY_ALREADY_INSERTED", "entry-already-inserted" },
-      { GDATA_SERVICE_ERROR_WITH_INSERTION, "GDATA_SERVICE_ERROR_WITH_INSERTION", "with-insertion" },
       { GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED, "GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED", "authentication-required" },
-      { GDATA_SERVICE_ERROR_WITH_UPDATE, "GDATA_SERVICE_ERROR_WITH_UPDATE", "with-update" },
-      { GDATA_SERVICE_ERROR_WITH_DELETION, "GDATA_SERVICE_ERROR_WITH_DELETION", "with-deletion" },
       { GDATA_SERVICE_ERROR_NOT_FOUND, "GDATA_SERVICE_ERROR_NOT_FOUND", "not-found" },
       { GDATA_SERVICE_ERROR_CONFLICT, "GDATA_SERVICE_ERROR_CONFLICT", "conflict" },
       { GDATA_SERVICE_ERROR_FORBIDDEN, "GDATA_SERVICE_ERROR_FORBIDDEN", "forbidden" },
-      { GDATA_SERVICE_ERROR_WITH_DOWNLOAD, "GDATA_SERVICE_ERROR_WITH_DOWNLOAD", "with-download" },
-      { GDATA_SERVICE_ERROR_WITH_UPLOAD, "GDATA_SERVICE_ERROR_WITH_UPLOAD", "with-upload" },
       { GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER, "GDATA_SERVICE_ERROR_BAD_QUERY_PARAMETER", "bad-query-parameter" },
+      { GDATA_SERVICE_ERROR_NETWORK_ERROR, "GDATA_SERVICE_ERROR_NETWORK_ERROR", "network-error" },
+      { GDATA_SERVICE_ERROR_PROXY_ERROR, "GDATA_SERVICE_ERROR_PROXY_ERROR", "proxy-error" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("GDataServiceError", values);
@@ -51,7 +67,7 @@ gdata_authentication_error_get_type (void)
   return etype;
 }
 
-/* enumerations from "gdata-parsable.h" */
+/* enumerations from "./gdata/gdata-parsable.h" */
 GType
 gdata_parser_error_get_type (void)
 {
