@@ -43,6 +43,7 @@
 #include "gdata-types.h"
 #include "gdata-private.h"
 #include "gdata-access-handler.h"
+#include "gdata-download-stream.h"
 
 static void gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface);
 static void gdata_documents_entry_finalize (GObject *object);
@@ -58,6 +59,7 @@ struct _GDataDocumentsEntryPrivate {
 	GTimeVal last_viewed;
 	gchar *document_id;
 	gboolean writers_can_invite;
+	gboolean is_deleted;
 	GDataAuthor *last_modified_by;
 };
 
@@ -66,6 +68,7 @@ enum {
 	PROP_LAST_VIEWED,
 	PROP_DOCUMENT_ID,
 	PROP_LAST_MODIFIED_BY,
+	PROP_IS_DELETED,
 	PROP_WRITERS_CAN_INVITE
 };
 
@@ -132,6 +135,20 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 					FALSE,
 					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
+	/**
+	 * GDataDocumentsEntry:is-deleted:
+	 *
+	 * Indicates whether the document entry has been deleted (moved to the trash). Deleted documents will only
+	 * appear in query results if the #GDataDocumentsQuery:show-deleted property is %TRUE.
+	 *
+	 * Since: 0.5.0
+	 **/
+	g_object_class_install_property (gobject_class, PROP_IS_DELETED,
+				g_param_spec_boolean ("is-deleted",
+					"Deleted?", "Indicates whether the document entry has been deleted.",
+					FALSE,
+					G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
 	/**
 	 * GDataDocumentsEntry:document-id
 	 *
@@ -157,12 +174,16 @@ gdata_documents_entry_class_init (GDataDocumentsEntryClass *klass)
 					"Last modified by", "Indicates the author of the last modification.",
 					GDATA_TYPE_AUTHOR,
 					G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
 }
 
 static void
 gdata_documents_entry_init (GDataDocumentsEntry *self)
 {
 	self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, GDATA_TYPE_DOCUMENTS_ENTRY, GDataDocumentsEntryPrivate);
+
+	/* Initialise the edited properties to the current time */
+	g_get_current_time (&(self->priv->edited));
 }
 
 static gboolean
@@ -180,11 +201,10 @@ gdata_documents_entry_access_handler_init (GDataAccessHandlerIface *iface)
 static gboolean
 parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_data, GError **error)
 {
-	GDataDocumentsEntry *self;
+	GDataDocumentsEntry *self = GDATA_DOCUMENTS_ENTRY (parsable);
 
-	self = GDATA_DOCUMENTS_ENTRY (parsable);
-
-	if (xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
+	if (gdata_parser_is_namespace (node, "http://www.w3.org/2007/app") == TRUE &&
+	    xmlStrcmp (node->name, (xmlChar*) "edited") == 0) {
 		xmlChar *edited = xmlNodeListGetString (doc, node->children, TRUE);
 		if (g_time_val_from_iso8601 ((gchar*) edited, &(self->priv->edited)) == FALSE) {
 			gdata_parser_error_not_iso8601_format (node, (gchar*) edited, error);
@@ -192,15 +212,8 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			return FALSE;
 		}
 		xmlFree (edited);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "lastViewed") == 0) {
-		xmlChar *last_viewed = xmlNodeListGetString (doc, node->children, TRUE);
-		if (g_time_val_from_iso8601 ((gchar*) last_viewed, &(self->priv->last_viewed)) == FALSE) {
-			gdata_parser_error_not_iso8601_format (node, (gchar*) last_viewed, error);
-			xmlFree (last_viewed);
-			return FALSE;
-		}
-		xmlFree (last_viewed);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "writersCanInvite") ==  0) {
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/docs/2007") == TRUE &&
+	           xmlStrcmp (node->name, (xmlChar*) "writersCanInvite") ==  0) {
 		xmlChar *writers_can_invite = xmlGetProp (node, (xmlChar*) "value");
 		if (xmlStrcmp (writers_can_invite, (xmlChar*) "true") == 0) {
 			self->priv->writers_can_invite = TRUE;
@@ -212,43 +225,58 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 			return FALSE;
 		}
 		xmlFree (writers_can_invite);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "resourceId") ==  0) {
-		gchar **document_id_parts;
-		xmlChar *resource_id;
+	} else if (gdata_parser_is_namespace (node, "http://schemas.google.com/g/2005") == TRUE) {
+		if (xmlStrcmp (node->name, (xmlChar*) "lastViewed") == 0) {
+			xmlChar *last_viewed = xmlNodeListGetString (doc, node->children, TRUE);
+			if (g_time_val_from_iso8601 ((gchar*) last_viewed, &(self->priv->last_viewed)) == FALSE) {
+				gdata_parser_error_not_iso8601_format (node, (gchar*) last_viewed, error);
+				xmlFree (last_viewed);
+				return FALSE;
+			}
+			xmlFree (last_viewed);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "deleted") ==  0) {
+			/* <gd:deleted> */
+			/* Note that it doesn't have any parameters, so we unconditionally set priv->is_deleted to TRUE */
+			self->priv->is_deleted = TRUE;
+		} else if (xmlStrcmp (node->name, (xmlChar*) "resourceId") ==  0) {
+			gchar **document_id_parts;
+			xmlChar *resource_id;
 
-		if (self->priv->document_id != NULL)
-			return gdata_parser_error_duplicate_element (node, error);
+			if (self->priv->document_id != NULL)
+				return gdata_parser_error_duplicate_element (node, error);
 
-		resource_id = xmlNodeListGetString (doc, node->children, TRUE);
-		if (resource_id == NULL || *resource_id == '\0') {
+			resource_id = xmlNodeListGetString (doc, node->children, TRUE);
+			if (resource_id == NULL || *resource_id == '\0') {
+				xmlFree (resource_id);
+				return gdata_parser_error_required_content_missing (node, error);
+			}
+
+			document_id_parts = g_strsplit ((gchar*) resource_id, ":", 2);
+			if (document_id_parts == NULL) {
+				gdata_parser_error_unknown_content (node, (gchar*) resource_id, error);
+				xmlFree (resource_id);
+				return FALSE;
+			}
 			xmlFree (resource_id);
-			return gdata_parser_error_required_content_missing (node, error);
-		}
 
-		document_id_parts = g_strsplit ((gchar*) resource_id, ":", 2);
-		if (document_id_parts == NULL) {
-			gdata_parser_error_unknown_content (node, (gchar*) resource_id, error);
-			xmlFree (resource_id);
-			return FALSE;
+			self->priv->document_id = g_strdup (document_id_parts[1]);
+			g_strfreev (document_id_parts);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "feedLink") ==  0) {
+			GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
+			if (link == NULL)
+				return FALSE;
+			gdata_entry_add_link (GDATA_ENTRY (self), link);
+			g_object_unref (link);
+		} else if (xmlStrcmp (node->name, (xmlChar*) "lastModifiedBy") ==  0) {
+			GDataAuthor *last_modified_by = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
+			if (last_modified_by == NULL)
+				return FALSE;
+			self->priv->last_modified_by = last_modified_by;
+		} else {
+			return GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 		}
-		xmlFree (resource_id);
-
-		self->priv->document_id = g_strdup (document_id_parts[1]);
-		g_strfreev (document_id_parts);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "feedLink") ==  0) {
-		GDataLink *link = GDATA_LINK (_gdata_parsable_new_from_xml_node (GDATA_TYPE_LINK, doc, node, NULL, error));
-		if (link == NULL)
-			return FALSE;
-		gdata_entry_add_link (GDATA_ENTRY (self), link);
-		g_object_unref (link);
-	} else if (xmlStrcmp (node->name, (xmlChar*) "lastModifiedBy") ==  0) {
-		GDataAuthor *last_modified_by = GDATA_AUTHOR (_gdata_parsable_new_from_xml_node (GDATA_TYPE_AUTHOR, doc, node, NULL, error));
-		if (last_modified_by == NULL)
-			return FALSE;
-		self->priv->last_modified_by = last_modified_by;
-	} else if (GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error) == FALSE) {
-		/* Error! */
-		return FALSE;
+	} else {
+		return GDATA_PARSABLE_CLASS (gdata_documents_entry_parent_class)->parse_xml (parsable, doc, node, user_data, error);
 	}
 
 	return TRUE;
@@ -290,6 +318,9 @@ gdata_documents_entry_get_property (GObject *object, guint property_id, GValue *
 		case PROP_WRITERS_CAN_INVITE:
 			g_value_set_boolean (value, priv->writers_can_invite);
 			break;
+		case PROP_IS_DELETED:
+			g_value_set_boolean (value, priv->is_deleted);
+			break;
 		case PROP_EDITED:
 			g_value_set_boxed (value, &(priv->edited));
 			break;
@@ -338,7 +369,6 @@ get_xml (GDataParsable *parsable, GString *xml_string)
 		g_string_append (xml_string, "<docs:writersCanInvite value='false'/>");
 }
 
-
 static void
 get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
 {
@@ -354,7 +384,7 @@ get_namespaces (GDataParsable *parsable, GHashTable *namespaces)
  * @edited: a #GTimeVal
  *
  * Gets the #GDataDocumentsEntry:edited property and puts it in @edited. If the property is unset,
- * both fields in the #GTimeVal will be set to %0.
+ * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
  *
  * Since: 0.4.0
  **/
@@ -372,7 +402,7 @@ gdata_documents_entry_get_edited (GDataDocumentsEntry *self, GTimeVal *edited)
  * @last_viewed: a #GTimeVal
  *
  * Gets the #GDataDocumentsEntry:last-viewed property and puts it in @last_viewed. If the property is unset,
- * both fields in the #GTimeVal will be set to %0.
+ * both fields in the #GTimeVal will be set to <code class="literal">0</code>.
  *
  * Since: 0.4.0
  **/
@@ -388,11 +418,15 @@ gdata_documents_entry_get_last_viewed (GDataDocumentsEntry *self, GTimeVal *last
  * gdata_documents_entry_get_path:
  * @self: a #GDataDocumentsEntry
  *
- * Gets the #GDataDocumentsEntry:path property.
+ * Builds a path for the #GDataDocumentsEntry, starting from a root node and traversing the folders containing the document, then
+ * ending with the document's ID.
  *
- * Note: the path is based on the entry ID, and not the entry human readable name (#GDataEntry::title).
+ * An example path would be: "/folder_id1/folder_id2/document_id".
  *
- * Return value: the folder hierarchy path containing the entry, or %NULL; free with g_free()
+ * Note: the path is based on the entry/document IDs of the folders (#GDataEntry:id) and document (#GDataDocumentsEntry:document-id),
+ * and not the entries' human-readable names (#GDataEntry:title).
+ *
+ * Return value: the folder hierarchy path containing the document, or %NULL; free with g_free()
  *
  * Since: 0.4.0
  **/
@@ -413,6 +447,9 @@ gdata_documents_entry_get_path (GDataDocumentsEntry *self)
 		gchar *folder_id = NULL;
 		gchar **link_href_cut = g_strsplit (gdata_link_get_uri (GDATA_LINK (element->data)), "/", 0);
 
+		/* Extract the folder ID from the folder URI, which is of the form:
+		 *   http://docs.google.com/feeds/documents/private/full/folder%3Afolder_id
+		 * We want the "folder_id" bit. */
 		for (i = 0;; i++) {
 			gchar **path_cut = NULL;
 
@@ -432,6 +469,7 @@ gdata_documents_entry_get_path (GDataDocumentsEntry *self)
 		g_strfreev (link_href_cut);
 		g_assert (folder_id != NULL);
 
+		/* Append the folder ID to our path */
 		g_string_append (path, folder_id);
 		g_string_append_c (path, '/');
 		g_free (folder_id);
@@ -511,10 +549,27 @@ gdata_documents_entry_get_last_modified_by (GDataDocumentsEntry *self)
 	return self->priv->last_modified_by;
 }
 
-static void
-got_chunk_cb (SoupMessage *message, SoupBuffer *chunk, GOutputStream *output_stream)
+/**
+ * gdata_documents_entry_is_deleted:
+ * @self: a #GDataDocumentsEntry
+ *
+ * Gets the #GDataDocumentsEntry:is-deleted property.
+ *
+ * Return value: %TRUE if the document has been deleted, %FALSE otherwise
+ *
+ * Since: 0.5.0
+ **/
+gboolean
+gdata_documents_entry_is_deleted (GDataDocumentsEntry *self)
 {
-	g_output_stream_write (output_stream, (void*) chunk->data, chunk->length, NULL, NULL);
+	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), FALSE);
+	return self->priv->is_deleted;
+}
+
+static void
+notify_content_type_cb (GDataDownloadStream *download_stream, GParamSpec *pspec, gchar **content_type)
+{
+	*content_type = g_strdup (gdata_download_stream_get_content_type (download_stream));
 }
 
 /*
@@ -523,7 +578,7 @@ got_chunk_cb (SoupMessage *message, SoupBuffer *chunk, GOutputStream *output_str
  * @service: an authenticated #GDataDocumentsService
  * @content_type: return location for the document's content type, or %NULL; free with g_free()
  * @download_uri: the URI to download the document
- * @destination_directory: the directory into which the file should be downloaded
+ * @destination_file: the #GFile into which the document file should be saved
  * @file_extension: the extension with which to save the downloaded file
  * @replace_file_if_exists: %TRUE if you want to replace the file if it exists, %FALSE otherwise
  * @cancellable: optional #GCancellable object, or %NULL
@@ -540,31 +595,35 @@ got_chunk_cb (SoupMessage *message, SoupBuffer *chunk, GOutputStream *output_str
  *
  * If @service isn't authenticated, a %GDATA_SERVICE_ERROR_AUTHENTICATION_REQUIRED is returned.
  *
- * If there is an error downloading the document, a %GDATA_SERVICE_ERROR_WITH_QUERY error will be returned.
+ * If there is an error downloading the document, a %GDATA_SERVICE_ERROR_PROTOCOL_ERROR error will be returned.
+ *
+ * If @destination_file is a directory, the file will be downloaded to this directory with the #GDataEntry:title and 
+ * the appropriate extension as its filename.
  *
  * Return value: a #GFile pointing to the downloaded document, or %NULL; unref with g_object_unref()
  *
  * Since: 0.4.0
  */
 GFile *
-_gdata_documents_entry_download_document (GDataDocumentsEntry *self, GDataService *service, gchar **content_type, gchar *download_uri,
-					  GFile *destination_directory, const gchar *file_extension, gboolean replace_file_if_exists,
+_gdata_documents_entry_download_document (GDataDocumentsEntry *self, GDataService *service, gchar **content_type, const gchar *src_uri,
+					  GFile *destination_file, const gchar *file_extension, gboolean replace_file_if_exists,
 					  GCancellable *cancellable, GError **error)
 {
-	GDataServiceClass *klass;
-	GFileOutputStream *file_stream;
-	GFile *destination_file;
-	SoupMessage *message;
-	guint status;
 	const gchar *document_title;
-	gchar *filename;
+	gchar *default_filename;
+	GFileOutputStream *dest_stream;
+	GInputStream *src_stream;
+	GFile *actual_file = NULL;
+	GError *child_error = NULL;
 
 	/* TODO: async version */
 	g_return_val_if_fail (GDATA_IS_DOCUMENTS_ENTRY (self), NULL);
 	g_return_val_if_fail (GDATA_IS_SERVICE (service), NULL);
-	g_return_val_if_fail (download_uri != NULL, NULL);
-	g_return_val_if_fail (G_IS_FILE (destination_directory), NULL);
+	g_return_val_if_fail (src_uri != NULL, NULL);
+	g_return_val_if_fail (G_IS_FILE (destination_file), NULL);
 	g_return_val_if_fail (file_extension != NULL, NULL);
+	g_return_val_if_fail (cancellable == NULL || G_IS_CANCELLABLE (cancellable), NULL);
+	g_return_val_if_fail (error == NULL || *error == NULL, NULL);
 
 	/* Ensure we're authenticated first */
 	if (gdata_service_is_authenticated (GDATA_SERVICE (service)) == FALSE) {
@@ -573,70 +632,28 @@ _gdata_documents_entry_download_document (GDataDocumentsEntry *self, GDataServic
 		return NULL;
 	}
 
-	/* Prepare the GFile */
+	/* Determine a default filename based on the document's title */
 	document_title = gdata_entry_get_title (GDATA_ENTRY (self));
-	filename = g_strdup_printf ("%s.%s", document_title, file_extension);
-	destination_file = g_file_get_child (destination_directory, filename);
-	g_free (filename);
+	default_filename = g_strdup_printf ("%s.%s", document_title, file_extension);
 
-	/* Check if the file exists */
-	if (g_file_query_exists (destination_file, cancellable) == TRUE) {
-		/* Replace a pre-existing file */
-		if (replace_file_if_exists == TRUE) {
-			file_stream = g_file_replace (destination_file, NULL, TRUE, G_FILE_CREATE_REPLACE_DESTINATION,
-						      cancellable, error);
-		} else {
-			g_set_error (error, G_IO_ERROR_EXISTS, 1, NULL);
-			return NULL;
-		}
-	} else {
-		/* Create a new file */
-		file_stream = g_file_create (destination_file, G_FILE_CREATE_NONE, cancellable, error);
-	}
+	dest_stream = _gdata_download_stream_find_destination (default_filename, destination_file, &actual_file, replace_file_if_exists, cancellable, error);
+	g_free (default_filename);
 
-	/* Get the document URI */
-	message = soup_message_new (SOUP_METHOD_GET, download_uri);
+	if (dest_stream == NULL)
+		return NULL;
 
-	/* We copy the data to disk as it comes through the network pipe */
-	soup_message_body_set_accumulate (message->response_body, FALSE);
-	g_signal_connect (message, "got-chunk", (GCallback) got_chunk_cb, file_stream);
-
-	/* Make sure the headers are set */
-	klass = GDATA_SERVICE_GET_CLASS (service);
-	if (klass->append_query_headers != NULL)
-		klass->append_query_headers (GDATA_SERVICE (service), message);
-
-	/* Send the message */
-	status = _gdata_service_send_message (GDATA_SERVICE (service), message, error);
-	g_object_unref (file_stream);
-	if (status == SOUP_STATUS_NONE) {
-		g_object_unref (message);
-		g_object_unref (destination_file);
+	/* Synchronously splice the data from the download stream to the file stream (network -> disk) */
+	src_stream = gdata_download_stream_new (GDATA_SERVICE (service), src_uri);
+	g_signal_connect (src_stream, "notify::content-type", (GCallback) notify_content_type_cb, content_type);
+	g_output_stream_splice (G_OUTPUT_STREAM (dest_stream), src_stream,
+				G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE | G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET, cancellable, &child_error);
+	g_object_unref (src_stream);
+	g_object_unref (dest_stream);
+	if (child_error != NULL) {
+		g_object_unref (actual_file);
+		g_propagate_error (error, child_error);
 		return NULL;
 	}
 
-	/* Check for cancellation */
-	if (g_cancellable_set_error_if_cancelled (cancellable, error) == TRUE) {
-		g_object_unref (message);
-		g_object_unref (destination_file);
-		return NULL;
-	}
-
-	if (status != 200) {
-		/* Error */
-		g_assert (klass->parse_error_response != NULL);
-		klass->parse_error_response (GDATA_SERVICE (service), GDATA_SERVICE_ERROR_WITH_QUERY, status, message->reason_phrase,
-					     message->response_body->data, message->response_body->length, error);
-		g_object_unref (message);
-		g_object_unref (destination_file);
-		return NULL;
-	}
-
-	/* Sort out the return values */
-	if (content_type != NULL)
-		*content_type = g_strdup (soup_message_headers_get_content_type (message->response_headers, NULL));
-
-	g_object_unref (message);
-
-	return destination_file;
+	return actual_file;
 }
